#!/usr/bin/env python
"""
`htrc.volumes`

Contains the configuration parser object.
"""
from future import standard_library
standard_library.install_aliases()
from builtins import input

from configparser import RawConfigParser as ConfigParser, NoSectionError
from codecs import open
from getpass import getpass
import logging
import os.path
import shutil
import time

from htrc.lib.cli import bool_prompt

DEFAULT_PATH = os.path.expanduser('~')
DEFAULT_PATH = os.path.join(DEFAULT_PATH, '.htrc')
if not os.path.exists(DEFAULT_PATH):
    DEFAULT_FILE = os.path.dirname(__file__)
    DEFAULT_FILE = os.path.join(DEFAULT_FILE, '.htrc.default')
    logging.info("Copying default config file to home directory.")
    shutil.copyfile(DEFAULT_FILE, DEFAULT_PATH)

def _get_value(section, key, path=None):
    if path is None:
        path = DEFAULT_PATH

    config = ConfigParser(allow_no_value=True)
    with open(path, encoding='utf8') as configfile:
        config.readfp(configfile)
    try:
        return config.get(section, key)
    except NoSectionError:
        raise EnvironmentError("Config not set for {} {} in {}".format(
            section, key, path))
    
def get_dataapi_port(path=None):
    port = int(_get_value('data', 'port', path))
    return (port)

def get_dataapi_host(path=None):
    host = _get_value('data', 'host', path)
    return (host)

def get_dataapi_epr(path=None):
    return _get_value('data', 'url', path)

def get_dataapi_cert(path=None):
    return _get_value('data', 'cert', path)

def get_dataapi_key(path=None):
    return _get_value('data', 'key', path)

def get_idp_host_port(path=None):
    host = _get_value('idp', 'host', path)
    port = _get_value('idp', 'port', path)

    return (host, port)

def get_idp_path(path=None):
    return _get_value('idp', 'url')

def get_idp_url(path=None):
    host, port = get_idp_host_port(path)
    path = get_idp_path(path)
    if port == 443:
        # On HTTPS Default Path
        return "https://{}{}".format(host, path)
    else:
        return "https://{}:{}{}".format(host, port, path)


# Add jwt credential access methods
def get_jwt_token(path=None):
    try:
        token = _get_value('jwt', 'token', path)

        # check expiration date
        expiration = int(_get_value('jwt', 'expiration', path))
        if time.time() > expiration:
            raise RuntimeError("JWT token expired.") 
    except:
        # This should run on either a missing or expired token.
        import htrc.auth
        token, expiration = htrc.auth.get_jwt_token()
        htrc.config.save_jwt_token(token, expiration, path)


    return token

def save_jwt_token(token, expiration=None, path=None):
    """
    Saves JWT token in the config file.
    """
    # Default to ~/.htrc
    if path is None:
        path = DEFAULT_PATH

    # Default to expiration of now - force a new token on next request
    if expiration is None:
        expiration = time.time()

    # Open and modify existing config file, if it exists.
    config = ConfigParser(allow_no_value=True)
    if os.path.exists(path):
        config.read(path)
    if not config.has_section('jwt'):
        config.add_section('jwt')

    # set token and expiration
    config.set('jwt', 'token', token)
    config.set('jwt', 'expiration', expiration)

    with open(path, 'w') as credential_file:
        config.write(credential_file)

    return token

def remove_jwt_token(path=None):
    """
    Removes JWT token from the config file.
    """
    # Default to ~/.htrc
    if path is None:
        path = DEFAULT_PATH

    # Open and modify existing config file, if it exists.
    config = ConfigParser(allow_no_value=True)
    if os.path.exists(path):
        config.read(path)
    if not config.has_section('jwt'):
        config.add_section('jwt')
    # set token and expiration
    config.set('jwt', 'token', " ")
    config.set('jwt', 'expiration', " ")

    with open(path, 'w') as credential_file:
        config.write(credential_file)


def get_credentials(path=None):
    """
    Retrieves the username and password from a config file for the Data API.
    Raises an EnvironmentError if not specified.
    See also: credential_prompt
    """
    client_id = _get_value('idp', 'client_id', path)
    client_secret = _get_value('idp', 'client_secret', path)

    if not client_id and not client_secret:
        logging.error("Config path: {}".format(path))
        raise EnvironmentError("No client_id and client_secret stored in config file.")

    return (client_id, client_secret)

def populate_parser(parser):
    return parser

if __name__ == '__main__':
    from argparse import ArgumentParser

    parser = ArgumentParser()
    parser = populate_parser(parser)
    parser.parse_args()




#!/usr/bin/env python
"""
Master script for HTRC Workset Toolkit.
"""
from __future__ import absolute_import, division, print_function
from future import standard_library
standard_library.install_aliases()

import json
import os, os.path
import shutil
import sys
from tempfile import NamedTemporaryFile

from htrc.metadata import get_metadata, get_volume_metadata
import htrc.volumes
import htrc.workset
import htrc.tools.mallet
from argparse import ArgumentParser
import htrc.tools.topicexplorer
from htrc.lib.cli import bool_prompt
from htrc.util.resolve import *


def download_parser(parser=None):
    if parser is None:
        parser = ArgumentParser()
    parser.add_argument("-u", "--username", help="HTRC username")
    parser.add_argument("-p", "--password", help="HTRC password")
    parser.add_argument("file", nargs='?', default=sys.stdin,
        help="workset path[s]")
    parser.add_argument("-f", "--force", action='store_true', 
        help="remove folder if exists")
    parser.add_argument("-o", "--output", help="output directory",
        default='/media/secure_volume/workset/')
    parser.add_argument("-c", "--concat", action='store_true',
        help="concatenate a volume's pages in to a single file")
    parser.add_argument("-pg", "--pages",action='store_true',
        help="Download given page numbers of a volumes.")
    parser.add_argument("-t", "--token", help="JWT for volumes download.")
    parser.add_argument("-dh", "--datahost", help="Data API host.")
    parser.add_argument("-dp", "--dataport", help="Data API port.")
    parser.add_argument("-de", "--dataepr", help="Data API EPR.")
    parser.add_argument("-dc", "--datacert", help="Client certificate file for mutual TLS with Data API.")
    parser.add_argument("-dk", "--datakey", help="Client key file for mutual TLS with Data API.")
    return parser

def add_workset_path(parser=None):
    if parser is None:
        parser = ArgumentParser()
    parser.add_argument("path", nargs='+', help="workset path[s]")
    return parser

    

def main():
    parser = ArgumentParser()
    parser.add_argument('-d', '--debug', help="print long debug messages",
                        action='store_true')
    parsers = parser.add_subparsers(help="select a command")

    # Metadata Helpers
    parser_getmd = parsers.add_parser('metadata',
                                      help="Get metadata for a folder of HathiTrust volumes")
    add_workset_path(parser_getmd)
    parser_getmd.set_defaults(func='metadata')

    # Export Helpers
    parser_export = parsers.add_parser('export',
                                      help="Export the list of HathiTrust volumes")
    add_workset_path(parser_export)
    parser_export.set_defaults(func='export')

    # Download Helper
    parser_download = parsers.add_parser('download',
        help="Download HathiTrust volumes to disk [requires auth]")
    download_parser(parser_download)
    parser_download.set_defaults(func='download')

    # Run helper
    parser_run = parsers.add_parser('run', help="Run a built-in algorithm.")
    run_parsers = parser_run.add_subparsers(help="select a command")

    parser_mallet = run_parsers.add_parser('mallet')
    htrc.tools.mallet.populate_parser(parser_mallet)
    parser_mallet.set_defaults(run='mallet')
    
    parser_topicexplorer = run_parsers.add_parser('topicexplorer')
    htrc.tools.topicexplorer.populate_parser(parser_topicexplorer)
    parser_topicexplorer.set_defaults(run='topicexplorer')
    
    parser_run.set_defaults(func='run')

    args = parser.parse_args()

    if args.func in ['metadata', 'export']:
        volumes = []
        if not args.path:
            for line in sys.stdin:
                volumes.append(line)
        else:
            for path in args.path:
                try:
                    volumes.extend(htrc.workset.path_to_volumes(path))
                except ValueError:
                    volumes.append(path)
        if args.func == 'export':
            for volume in volumes:
                print(volume)
        if args.func == 'metadata':
            metadata = get_metadata(volumes)
            print(json.dumps(metadata))
    elif args.func == 'run':
        if args.run == 'mallet':
            htrc.tools.mallet.main(args.path, args.k, args.iter)
        if args.run == 'topicexplorer':
            htrc.tools.topicexplorer.main(args.path, args.k, args.iter)
    elif args.func == 'download':
        if os.path.exists(args.output):
            if args.force or bool_prompt('Folder {} exists. Delete?'.format(args.output), default=False):
                shutil.rmtree(args.output)
                os.makedirs(args.output)
            else:
                print("Please choose another output folder and try again.")
                sys.exit(1)

        try:
            resolve_and_download(args)
        except ValueError:
            print("Invalid identifier:", args.file)
            sys.exit(1)

def resolve_and_download(args):
    if args.file == sys.stdin:
        # For use with UNIX pipes
        download_with_tempfile(args, sys.stdin)
        return

    elif os.path.exists(args.file):
        # For use with downloaded workset files - either in JSON or 
        download(args)
        return

    elif (args.file.endswith('json')
        or args.file.endswith('jsonld')
        or args.file.startswith('http://')
        or args.file.startswith('https://')):
        # For use with HTRC Worksets and HT Collection Builder
        try:
            volumes = htrc.workset.load(args.file)
            download_with_tempfile(args, volumes)
            return
        except ValueError:
            # Invalid workset, continue to last block
            pass

    # Check for valid volume_id
    try:
        if parse_volume_id(args.file):
            volumes = [parse_volume_id(args.file)]
            download_with_tempfile(args, volumes)
            return
        else:
            raise ValueError("No Volume ID found")
    except ValueError:
        pass
    
    # Check for valid record id
    if parse_record_id(args.file):
        record_id = parse_record_id(args.file)
        volumes = record_id_to_volume_ids(record_id)
        download_with_tempfile(args, volumes)
        return
    else:
        # invalid
        raise ValueError("Not a valid ID file or workset identifier: {}".format(
                         args.file))


def download(args):
    try:
        htrc.volumes.download(args)
    except OSError as e:
        if not os.path.exists('/media/secure_volume/'):
            print('Secure volume not mounted. Could not download volumes')
            sys.exit(1)
        else:
            print("Could not download volumes. {} {}".format(e.strerror, e.filename))
            sys.exit(1)
    except RuntimeError as e:
        if not args.debug:
            print("Could not download volumes. {}".format(str(e)))
            sys.exit(1)
        else:
            raise e

def download_with_tempfile(args, volumes):
    f = NamedTemporaryFile()
    for volume in volumes:
        f.write((volume + '\n').encode('utf-8'))
    f.flush()
    args.file = f.name

    try:
        download(args)
    finally:
        print("Closing temporary file: " + f.name)
        f.close()


if __name__ == '__main__':
    main()



from __future__ import print_function
from future import standard_library
standard_library.install_aliases()

import sys
if sys.version_info.major == 2:
    from mock import Mock, patch, PropertyMock
elif sys.version_info.major == 3:
    from unittest.mock import Mock, patch, PropertyMock

from io import BytesIO  # used to stream http response into zipfile.
from tempfile import NamedTemporaryFile, mkdtemp
import unittest2 as unittest

import htrc.mock.volumes

class MockResponse(BytesIO):
    def __init__(self, data, status=200, *args, **kwargs):
        BytesIO.__init__(self, data, *args, **kwargs)
        self.status = status

class TestVolumes(unittest.TestCase):
    def setUp(self):
        self.test_vols = ['mdp.39015050817181', 'mdp.39015055436151',
            'mdp.39015056169157', 'mdp.39015050161697', 'mdp.39015042791874']

        self.config_path = NamedTemporaryFile(delete=False).name
        self.empty_config_path = NamedTemporaryFile(delete=False).name

        self.output_path = mkdtemp()

    def tearDown(self):
        import os, shutil
        os.remove(self.config_path)
        shutil.rmtree(self.output_path)

    @patch('htrc.mock.volumes.credential_prompt')
    def test_credential_prompt(self, credential_prompt_mock):
        # configure mocks
        credential_prompt_mock.return_value = ('1234', '1234')

        # test prompts
        username, password = htrc.mock.volumes.credential_prompt(self.config_path)
        self.assertEqual(username, '1234')
        self.assertEqual(password, '1234')

        # test read
        username, password = htrc.mock.volumes.credentials_from_config(
            self.config_path)
        self.assertEqual(username, None)
        self.assertEqual(password, None)

    def test_get_oauth2_token(self):
        token = htrc.mock.volumes.get_oauth2_token('1234','1234')
        self.assertEqual(token, 'a1b2c3d4e5f6')

    def test_get_volumes_and_pages(self):
        htrc.mock.volumes.get_volumes('1234', self.test_vols)
        htrc.mock.volumes.get_pages('1234', self.test_vols)

    def test_get_volumes_and_pages_empty(self):
        with self.assertRaises(ValueError):
            htrc.mock.volumes.get_volumes('1234', [])

        with self.assertRaises(ValueError):
            htrc.mock.volumes.get_pages('1234', [])

    @patch('htrc.mock.volumes.ZipFile')
    @patch('htrc.mock.volumes.get_volumes')
    @patch('htrc.mock.volumes.get_oauth2_token')
    def test_download_volumes(self, oauth2_mock, volumes_mock, zip_mock):
        oauth2_mock.return_value = 'a1b2c3d4e5'
        volumes_mock.return_value = b''

        htrc.mock.volumes.download_volumes(self.test_vols, self.output_path,
            username='1234', password='1234')

        # test directory creation
        import shutil
        shutil.rmtree(self.output_path)
        htrc.mock.volumes.download_volumes(self.test_vols, self.output_path,
            username='1234', password='1234')

    @patch('htrc.mock.volumes.ZipFile')
    @patch('htrc.mock.volumes.get_volumes')
    @patch('htrc.mock.volumes.get_oauth2_token')
    def test_download_volumes_saved_creds(self, oauth2_mock, volumes_mock, zip_mock):
        oauth2_mock.return_value = 'a1b2c3d4e5'
        volumes_mock.return_value = b''

        # test config-based auth
        import os, os.path
        config_path = os.path.expanduser('~')
        config_path = os.path.join(config_path, '.htrc')
        preexisting_config = os.path.exists(config_path)
        if not preexisting_config:
            htrc.mock.volumes.save_credentials('1234', '1234', config_path)

        htrc.mock.volumes.download_volumes(self.test_vols, self.output_path)

        if not preexisting_config:
            os.remove(config_path)

    def test_download(self):
        pass

suite = unittest.TestLoader().loadTestsFromTestCase(TestVolumes)
unittest.TextTestRunner(verbosity=2).run(suite)




from builtins import str
import os, os.path
import subprocess
import sys
import tarfile
import wget

from htrc.volumes import download_volumes
from htrc.workset import path_to_volumes

MALLET_DIR = os.path.expanduser('~/mallet')

# Mallet is downloaded and intalled in user's home directory
def install_mallet():
    if not os.path.exists(MALLET_DIR):
        os.makedirs(MALLET_DIR)
        mallet_zip = wget.download('http://mallet.cs.umass.edu/dist/mallet-2.0.8RC3.tar.gz')
        mallet_dir = tarfile.open(mallet_zip, "r:gz")
        mallet_dir.extractall(path=MALLET_DIR)
        mallet_dir.close()

def main(path, topics, iterations, output_dir='/media/secure_volume/workset/'):
    if not os.path.exists(MALLET_DIR):
        if not os.path.exists('/media/secure_volume/'):
            print('Installing Mallet ...')
            install_mallet()
            print('\n')
        else:
            print('Mallet not installed, but capsule is in secure mode.')
            print('Switch to maintenance mode and run this command again')
            print('to install Mallet. Then, switch to secure mode to train')
            print('topic models.')
            sys.exit(1)

    if not os.path.isdir(path):
        try:
            volumes = path_to_volumes(path)
        except ValueError as e:
            print("Could not process workset. {}".format(str(e)))
            sys.exit(1)

        try:
            download_volumes(volumes, output_dir)
        except OSError as e:
            if not os.path.exists('/media/secure_volume/'):
                print('Secure volume not mounted. Could not download volumes')
                sys.exit(1)
            else:
                print("Could not download volumes. {} {}".format(e.strerror, e.filename))
                sys.exit(1)
        except RuntimeError as e:
            if not args.debug:
                print("Could not download volumes. {}".format(str(e)))
                sys.exit(1)
            else:
                raise e
        path = output_dir


    # import the workset to MALLET format.
    subprocess.check_call([
        '{}/mallet-2.0.8RC3/bin/mallet'.format(MALLET_DIR),
        'import-dir',
        '--input', path,
        '--output', os.path.join(path, '../corpus.mallet'),
        '--keep-sequence',
        '--remove-stopwords'
        ])

    subprocess.check_call([
        '{}/mallet-2.0.8RC3/bin/mallet'.format(MALLET_DIR),
        'train-topics',
        '--input', os.path.join(path, '../corpus.mallet'),
        '--num-topics', str(topics),
        '--output-state', os.path.join(path, '../mallet_state.gz'),
        '--output-topic-keys', os.path.join(path, '../mallet_topic-keys.txt'),
        '--output-doc-topics', os.path.join(path, '../mallet_doc-topics.txt'),
        '--num-iterations', str(iterations)
        ])

def populate_parser(parser=None):
    if parser is None:
        from argparse import ArgumentParser
        parser = ArgumentParser()
    parser.add_argument('-k', help="number of topics", required=True)
    parser.add_argument('--iter', help="number of iterations", default=200)
    parser.add_argument('--workset-path', help="Location to store workset download.",
                        default='/media/secure_volume/workset/')
    parser.add_argument('path', default='/media/secure_volume/workset/',
        nargs='?')
    return parser


if __name__ == '__main__':
    from argparse import ArgumentParser
    parser = ArgumentParser(description="MALLET tools for the HTRC")
    populate_parser(parser)
    args = parser.parse_args()

    main(args.path, args.k, args.iter, args.workset_path)



#!/usr/bin/env python
"""
`htrc.volumes`

Contains functions to retrieve volumes from the HTRC Data API. 

The functions in this package will not operate unless they are 
executed from an HTRC Data Capsule in Secure Mode. The module 
`htrc.mock.volumes` contains Patch objects for testing workflows.
"""
from __future__ import print_function
from future import standard_library
standard_library.install_aliases()

from builtins import input

import http.client
from io import BytesIO  # used to stream http response into zipfile.
import json
import logging
import os.path
import progressbar
import re
import socket
import ssl
import sys
from time import sleep
from urllib.request import urlopen
from urllib.error import HTTPError
from urllib.parse import quote_plus, urlencode
import xml.etree.ElementTree as ET
from zipfile import ZipFile  # used to decompress requested zip archives.

from htrc.lib.cli import bool_prompt
from htrc.util import split_items
import htrc.config

import logging
from logging import NullHandler
logging.getLogger(__name__).addHandler(NullHandler())

def get_volumes(token, volume_ids, host, port, cert, key, epr, concat=False):
    """
    Returns volumes from the Data API as a raw zip stream.

    Parameters:
    :token: An OAuth2 token for the app.
    :volume_ids: A list of volume_ids
    :concat: If True, return a single file per volume. If False, return a single
    file per page (default).
    :host: Data API host
    :port: Data API port
    """
    if not volume_ids:
        raise ValueError("volume_ids is empty.")

    url = epr + "volumes"

    for id in volume_ids:
        if ("." not in id
            or " " in id):
            print("Invalid volume id " + id + ". Please correct this volume id and try again.")

    data = {'volumeIDs': '|'.join(
        [id.replace('+', ':').replace('=', '/') for id in volume_ids])}
    if concat:
        data['concat'] = 'true'

    # Authorization
    headers = {"Authorization": "Bearer " + token,
               "Content-type": "application/x-www-form-urlencoded"}

    # Create SSL lookup
    # TODO: Fix SSL cert verification
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    # Retrieve the volumes
    httpsConnection = http.client.HTTPSConnection(host, port, context=ctx, key_file=key, cert_file=cert)


    httpsConnection.request("POST", url, urlencode(data), headers)

    response = httpsConnection.getresponse()

    if response.status is 200:
        body = True
        data = BytesIO()
        bytes_downloaded = 0
        bar = progressbar.ProgressBar(max_value=progressbar.UnknownLength,
            widgets=[progressbar.AnimatedMarker(), '    ',
                     progressbar.DataSize(),
                     ' (', progressbar.FileTransferSpeed(), ')'])

        while body:
            body = response.read(128)
            data.write(body)
            bytes_downloaded += len(body)
            bar.update(bytes_downloaded)

        data = data.getvalue()
    else:
        logging.debug("Unable to get volumes")
        logging.debug("Response Code: {}".format(response.status))
        logging.debug("Response: {}".format(response.reason))
        raise EnvironmentError("Unable to get volumes.")

    if httpsConnection is not None:
        httpsConnection.close()

    return data


def get_pages(token, page_ids, host, port, cert, key, epr, concat=False):
    """
    Returns a ZIP file containing specfic pages.

    Parameters:
    :token: An OAuth2 token for the app.
    :volume_ids: A list of volume_ids
    :concat: If True, return a single file per volume. If False, return a single
    file per page (default).
    """
    if not page_ids:
        raise ValueError("page_ids is empty.")

    url = epr + "pages"

    for id in page_ids:
        if ("." not in id
            or " " in id):
            print("Invalid volume id " + id + ". Please correct this volume id and try again.")

    data = {'pageIDs': '|'.join(
        [id.replace('+', ':').replace('=', '/') for id in page_ids])}

    if concat:
        data['concat'] = 'true'

    # Authorization
    headers = {"Authorization": "Bearer " + token,
               "Content-type": "application/x-www-form-urlencoded"}


    # Create SSL lookup
    # TODO: Fix SSL cert verification
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    # Retrieve the volumes
    httpsConnection = http.client.HTTPSConnection(host, port, context=ctx, key_file=key, cert_file=cert)


    httpsConnection.request("POST", url, urlencode(data), headers)

    response = httpsConnection.getresponse()

    if response.status is 200:
        body = True
        data = BytesIO()
        bytes_downloaded = 0
        bar = progressbar.ProgressBar(max_value=progressbar.UnknownLength,
              widgets=[progressbar.AnimatedMarker(), '    ',
                                               progressbar.DataSize(),
                                               ' (', progressbar.FileTransferSpeed(), ')'])

        while body:
            body = response.read(128)
            data.write(body)
            bytes_downloaded += len(body)
            bar.update(bytes_downloaded)

        data = data.getvalue()
    else:
        logging.debug("Unable to get pages")
        logging.debug("Response Code: ".format(response.status))
        logging.debug("Response: ".format(response.reason))
        raise EnvironmentError("Unable to get pages.")

    if httpsConnection is not None:
        httpsConnection.close()

    return data

def get_oauth2_token(username, password):
    # make sure to set the request content-type as application/x-www-form-urlencoded
    headers = {"Content-type": "application/x-www-form-urlencoded"}
    data = { "grant_type": "client_credentials",
             "client_secret": password,
             "client_id": username }
    data = urlencode(data)

    # create an SSL context
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    # make sure the request method is POST
    host, port = htrc.config.get_oauth2_host_port()
    oauth2port = htrc.config.get_oauth2_port()
    oauth2EPRurl = htrc.config.get_oauth2_url()
    httpsConnection = http.client.HTTPSConnection(host, oauth2port, context=ctx)
    httpsConnection.request("POST", oauth2EPRurl + "?" + data, "", headers)

    response = httpsConnection.getresponse()

    # if response status is OK
    if response.status == 200:
        data = response.read().decode('utf8')

        jsonData = json.loads(data)
        logging.info("*** JSON: {}".format(jsonData))

        token = jsonData["access_token"]
        logging.info("*** parsed token: {}".format(token))

    else:
        logging.debug("Unable to get token")
        logging.debug("Response Code: {}".format(response.status))
        logging.debug("Response: {}".format(response.reason))
        logging.debug(response.read())
        raise EnvironmentError("Unable to get token.")

    if httpsConnection is not None:
        httpsConnection.close()

    return token

def grep(file_name, pattern):
    print("\nFollowing volume ids are not available.")
    for line in open(file_name):
        if pattern in line:
            print (line.split()[-1])

def check_error_file(output_dir):
    file_name = "ERROR.err"

    if output_dir.endswith("/"):
        file_path = output_dir+ file_name
    else:
        file_path = output_dir+"/"+file_name

    if os.path.isfile(file_path):
        grep(file_path,"KeyNotFoundException")


def download_volumes(volume_ids, output_dir, username=None, password=None,
                     config_path=None, token=None, concat=False, pages=False, host=None, port=None, cert=None, key=None, epr=None):
    # create output_dir folder, if nonexistant
    if not os.path.isdir(output_dir):
        os.makedirs(output_dir)

    # get token if not specified
    if not token:
        token = htrc.config.get_jwt_token()
        htrc.config.remove_jwt_token()

    if not host:
        host= htrc.config.get_dataapi_host()

    if not port:
        port = htrc.config.get_dataapi_port()

    if not epr:
        epr = htrc.config.get_dataapi_epr()

    if not cert:
        cert = htrc.config.get_dataapi_cert()

    if not key:
        key = htrc.config.get_dataapi_key()

    if any((token, host, port)) is not None:
        logging.info("obtained token: %s\n" % token)

        try:
            for ids in split_items(volume_ids, 250):
                if pages:
                    data = get_pages(token, ids, host, port, cert, key, epr, concat)
                else:
                    data = get_volumes(token, ids, host, port, cert, key, epr, concat)

                myzip = ZipFile(BytesIO(data))
                myzip.extractall(output_dir)
                myzip.close()

                check_error_file(output_dir)

        except socket.error:
            raise RuntimeError("Data API request timeout. Is your Data Capsule in Secure Mode?")

    else:
        raise RuntimeError("Failed to obtain jwt token.")


def download(args):
    # extract files
    with open(args.file) as IDfile:
        volumeIDs = [line.strip() for line in IDfile]

    return download_volumes(volumeIDs, args.output,
        username=args.username, password=args.password,
        token=args.token, concat=args.concat, pages=args.pages, host=args.datahost,
        port=args.dataport, cert=args.datacert, key=args.datakey,
        epr=args.dataepr)




"""
MARC CODE HANDLING
"""
from __future__ import print_function
from future import standard_library
standard_library.install_aliases()
from builtins import str

import xml.etree.ElementTree as ET


def parse_marc(raw):
    # lazy workaround
    raw = raw.replace(' xmlns', ' xmlnamespace')
    ET.register_namespace('', 'http://www.loc.gov/MARC21/slim')
    return ET.fromstring(raw)


def get_marc_value(xml, tag, code):
    xpath = "{marc}datafield[@tag='{tag}']/{marc}subfield[@code='{code}']".format(
        tag=tag, code=code, marc='')  # marc="{http://www.loc.gov/MARC21/slim}")
    results = xml.findall(xpath)
    return results[0].text if results else None


def get_lccn_from_marc(xml):
    return get_marc_value(xml, '010', 'a')


def get_title_from_marc(xml):
    return get_marc_value(xml, '245', 'a')


def get_volume_from_marc(xml):
    return get_marc_value(xml, '974', 'c')


def get_lcc_from_marc(xml):
    # MARC tag 050a/b or 991h/i
    lcc = list()
    val = get_marc_value(xml, '050', 'a')
    if val:
        lcc.append(val)

    val = get_marc_value(xml, '050', 'b')
    if val:
        lcc[-1] += val

    val = get_marc_value(xml, '991', 'h')
    if val:
        lcc.append(val)

    val = get_marc_value(xml, '991', 'i')
    if val:
        lcc[-1] += val

    return ";".join(lcc)





from base64 import b64encode
from getpass import getpass
import http.client
import ssl
import time

import requests
import requests.auth

import htrc.config

def get_jwt_token():
    # Currently we just store one common jwt token locally at .htrc file for simplicity
    # Expect to add POST method to query unique jwt token with the combo of username and password
    username, password = credential_prompt()

    client_id, client_secret = htrc.config.get_credentials()

    auth = requests.auth.HTTPBasicAuth(client_id, client_secret)
    data = { "grant_type": "password",
             "username": username,
             "password": password,
             "scope" : "openid"}

    url = htrc.config.get_idp_url()
    r = requests.post(url, data=data, auth=auth)

    data = r.json()
    if 'error' not in data:
        expiration = int(time.time()) + data['expires_in']
        return data['id_token'], expiration
    elif data['error'] == 'invalid_grant':
        print("Invalid username or password. Please try again.\n")
        return get_jwt_token()
    else:
        raise RuntimeError("JWT token retrieval failed: {}".format(data['error']))

def credential_prompt():
    """
    A prompt for entering HathiTrust Research Center credentials.
    """
    print("Please enter your HathiTrust Research Center credentials.")
    username = input("HTRC Username: ")
    password = getpass("HTRC Password: ")

    if not username or not password:
        print("Invalid username or password. Please try again.\n")
        return credential_prompt()
    else:
        return (username, password)


if __name__ == '__main__':
    token, expiration = get_jwt_token()
    htrc.config.save_jwt_token(token, expiration)
